<?xml version="1.0" encoding="utf-8"?>

<!-- Dieser Prototyp eines XSLT-Skripts wurde im Rahmen des Lektürekurses „Digital Humanities – Digitale Methoden in der Musikwissenschaft“ im Sommersemester 2018 an der Ludwig-Maximilians-Universität München erstellt. Die Weiterverwendung bedarf einer Zustimmung durch den Kursleiter. Kontakt: dominik.leipold@lmu.de -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:tei="http://www.tei-c.org/ns/1.0"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	version="1.0" exclude-result-prefixes="xsl tei xs">
	<!-- XSLT-Stylesheet zur Transformation der XML-Datei für einen TEI-Textvergleich
		in ein HTML-Dokument zur synoptischen Darstellung -->
	<!-- Version: 1.2 -->

	<xsl:output method="html"/>

	<!-- erstelle Liste von Definitionen der relevanten Textzeugen -->
	<xsl:variable name="witnessDefs" select="//tei:teiHeader//tei:listWit/tei:witness"/>

	<xsl:template match="tei:TEI">
		<html>
			<head>
				<style>
					.tei-app {
						background-color: yellow;
					}
					.ed {
						color: red;
					}
					.conversion-error {
						background-color: orange;
					}
					td {
						width: 50%;
					}
				</style>
			</head>
			<body>
				<xsl:apply-templates select="tei:text"/>
			</body>
		</html>
	</xsl:template>

	<!-- verarbeite Inhalte innerhalb folgender TEI-Strukturelemente -->
	<xsl:template match="tei:text|tei:body">
		<xsl:apply-templates select="node()"/>
	</xsl:template>

	<!-- entferne PIs und Kommentare -->
	<xsl:template match="processing-instruction()|comment()"/>

	<!-- markiere Inhalt von Elementen, die von keiner Template behandelt werden -->
	<xsl:template match="*">
		<span class="conversion-error">
			<code class="ed">
				<xsl:value-of select="concat('&lt;',name(),'&gt;')"/>
			</code>
			<xsl:apply-templates select="node()"/>
			<code class="ed">
				<xsl:value-of select="concat('&lt;/',name(),'&gt;')"/>
			</code>
		</span>
	</xsl:template>

	<!-- Behandlung von text()-Knoten -->
	<xsl:template match="text()">
		<xsl:choose>
			<xsl:when test="ancestor-or-self::tei:head|ancestor-or-self::tei:l|ancestor-or-self::tei:p|ancestor-or-self::tei:speaker|ancestor-or-self::tei:stage">
				<!-- Text rendern -->
				<xsl:value-of select="."/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="normalize-space(.)"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Strukturelemente -->

	<xsl:template match="tei:div">
		<table class="synopsis">
			<thead>
				<tr>
					<xsl:for-each select="$witnessDefs">
						<th>
							<xsl:value-of select="./text()"/>
						</th>
					</xsl:for-each>
				</tr>
			</thead>
			<xsl:call-template name="separator"/>
			<xsl:apply-templates select="node()"/>
		</table>
	</xsl:template>

	<xsl:template match="tei:sp">
		<xsl:call-template name="separator"/>
		<xsl:apply-templates select="*"/>
	</xsl:template>

	<!-- Template zum Einfügen von Standard-Abstandshaltern -->
	<xsl:template name="separator">
		<tr class="separator">
			<xsl:for-each select="$witnessDefs">
				<td/>
			</xsl:for-each>
		</tr>
	</xsl:template>

	<!-- Inline-Elemente -->

	<xsl:template match="tei:byline|tei:head|tei:l|tei:p|tei:speaker|tei:stage">
		<xsl:if test="preceding-sibling::tei:sp">
			<xsl:call-template name="separator"/>
		</xsl:if>
		<tr class="tei-{name()}">
			<xsl:variable name="element" select="."/>
			<xsl:for-each select="$witnessDefs">
				<xsl:variable name="witnessId" select="./@xml:id"/>
				<td>
					<!-- stelle vor Weiterverarbeitung fest, ob mixed content vorliegt;
						falls nicht, ignoriere alle direkten text()-Nachfolger -->
					<!-- Spezifikation hier: Ein Element enthält Mixed Content, wenn
						eines der text()-Kinder mehr als nur Whitespace enthält oder
						aus genau einem Leerzeichen besteht (denn in diesem Fall muss
						dieses als Teil des Texts betrachtet werden). -->
					<xsl:choose>
						<xsl:when test="$element/text()[.=' ' or normalize-space(.)]">
							<xsl:apply-templates select="$element/node()">
								<xsl:with-param name="witnessId" select="$witnessId"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="$element/*">
								<xsl:with-param name="witnessId" select="$witnessId"/>
							</xsl:apply-templates>
						</xsl:otherwise>
					</xsl:choose>
				</td>
			</xsl:for-each>
		</tr>
	</xsl:template>

	<!-- spezielle Elemente und Formatierungen -->

	<xsl:template match="tei:gap">
		<span class="ed">[…]</span>
	</xsl:template>

	<!-- Lesarten -->

	<!-- nur app-Elemente mit type-Attribut erhalten eine HTML-Klasse;
		falls dieses fehlt, soll sich der Unterschied nicht hervorheben lassen -->

	<xsl:template match="tei:app">
		<xsl:param name="witnessId"/>
		<xsl:variable name="class" select="'tei-app'"/>
		<span class="{$class}">
			<xsl:apply-templates select="*">
				<xsl:with-param name="witnessId" select="$witnessId"/>
			</xsl:apply-templates>
		</span>
	</xsl:template>

	<xsl:template match="tei:rdg">
		<xsl:param name="witnessId"/>
		<xsl:if test="@wit=concat('#',$witnessId)">
			<xsl:apply-templates select="node()"/>
		</xsl:if>
	</xsl:template>

</xsl:stylesheet>
